/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.reporting;

import java.util.Iterator;
import java.util.List;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.util.config.Property;

/**
 * <summary> BasicStatsReporting is a reporting implementation that renders user-friendly reports. </summary>
 *
 * <configuration> It does not support any configuration. The constructor is included to address reflection
 * contract. </configuration>
 *
 * <thread-safety> This class is not thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class BasicStatsReporting extends AbstractStatsReporting {

    /**
     * <summary> Default constructor. Does nothing. </summary>
     */
    public BasicStatsReporting() {
        // your code here
    }

    /**
     * <summary> Configures from a property. Simply propagate it to super constructor. </summary>
     *
     * @param property the property to load config from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    public BasicStatsReporting(Property property) throws ConfigurationException {
        super(property);
    }

    /**
     * <summary> This generates a user-readable report. Please include info for everything hold on the reporting in
     * reasonable format. </summary>
     *
     * @return the generated report
     */
    public String generateReport() {
        StringBuffer sb = new StringBuffer();
        if (getBaseDirectory() != null) {
            sb.append("Base directory: " + getBaseDirectory() + "\n");
        }
        writeStatsCollection(sb, getFileStats(), "File");
        writeStatsCollection(sb, getDirectoryStats(), "Directory");
        writeStatsCollection(sb, getAliasStats(), "Alias");
        sb.append("Overall ");
        writeStats(sb, getOverallStats());
        sb.append('\n');
        return sb.toString();
    }

    /**
     * <summary> Writes stats collection to given string buffer in user readable format. Provided item name is used
     * to denote the stats collection elements. </summary>
     *
     * <p>The format for each item is <i>File filename stat=value, stat2=value2</i></p>
     *
     * @param sb the StringBuffer to write to
     * @param stats StatsCollection to be written
     * @param itemName string with name to be used with each collection item, e.g. File, Directory, Alias
     */
    private void writeStatsCollection(StringBuffer sb, StatsCollection stats, String itemName) {
        List keys = stats.getAllKeys();
        if (keys.isEmpty()) {
            sb.append(itemName + " collection is empty");
        }
        // each item is written in a seperate line
        for (Iterator iter = keys.iterator(); iter.hasNext();) {
            String key = (String) iter.next();
            sb.append(itemName + ' ' + key + ' ');
            writeStats(sb, stats.get(key));
            sb.append('\n');
        }
    }

    /**
     * <summary> Writes stats to given string buffer in user readable format, that is, comma separated
     * "name=value". </summary>
     *
     * @param sb the StringBuffer to write to
     * @param stats StatsResult to be written
     */
    private void writeStats(StringBuffer sb, StatsResult stats) {
        List metrics = stats.getMetrics();
        boolean first = true;
        for (Iterator iter = metrics.iterator(); iter.hasNext();) {
            Metric metric = (Metric) iter.next();
            // write comma, but for the first item
            if (first) {
                first = false;
            } else {
                sb.append(", ");
            }
            sb.append(metric.getDisplayName() + '=' + stats.getStat(metric));
        }
    }
}
