/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics;

import junit.framework.TestCase;

import com.topcoder.util.config.ConfigManager;

/**
 * <p>Unit test case for Main class. Actually, it is hard to test and verify the correctness of the Main.main()
 * method, but it is run with some arguments and no error is thrown. </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class MainTests extends TestCase {

    /**
     * Sets up the test case, load the default namespace.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        ConfigManager.getInstance()
                .add(Main.NAMESPACE, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);
    }

    /**
     * Disposes the test case, removes configuration namespace.
     *
     * @throws Exception to JUnit
     */
    protected void tearDown() throws Exception {
        ConfigManager cm = ConfigManager.getInstance();
        if (cm.existsNamespace(Main.NAMESPACE)) {
            cm.removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * <p>Invokes Main.main(String[] args) method.</p>
     *
     * @throws Exception to JUnit
     */
    public void testCommandLineDemo1() throws Exception {

        // this test requires the Main.NAMESPACE not to be loaded
        ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);

        // specify configuration file
        // -c conf/FileStatistics.xml test_files/a.txt test_files/b.jav
        String[] args = new String[] {"-c", "conf/FileStatistics.xml", "-o", "test_files/output.html",
            "test_files/a.txt", "test_files/b.jav"};
        Main.main(args);
    }

    /**
     * <p>Invokes Main.main(String[] args) method.</p>
     *
     * @throws Exception to JUnit
     */
    public void testCommandLineDemo2() throws Exception {
        // specify namespace
        // -n com.topcoder.file.statistics test_files/a.txt test_files/b.jav
        String namespace = "com.topcoder.file.statistics";

        // the namespace must be preloaded
        ConfigManager.getInstance().add(namespace, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);

        try {
            Main.main(new String[] {"-n", "com.topcoder.file.statistics", "-o", "test_files/output.html",
                "test_files/a.txt", "test_files/b.jav"});

        } finally {
            ConfigManager.getInstance().removeNamespace(namespace);
        }
    }

    /**
     * <p>Invokes Main.main(String[] args) method.</p>
     *
     * @throws Exception to JUnit
     */
    public void testCommandLineDemo3() throws Exception {
        // specify base directory
        // -b test_files test_files/a.txt test_files/b.jav

        Main.main(new String[] {"-b", "test_files", "-o", "test_files/output.html", "test_files/a.txt",
            "test_files/b.jav"});
    }

    /**
     * <p>Invokes Main.main(String[] args) method.</p>
     *
     * @throws Exception to JUnit
     */
    public void testCommandLineDemo4() throws Exception {
        // recursively process sub-directory
        // -r test_files
        Main.main(new String[] {"-r", "-o", "test_files/output.html", "test_files"});
    }

    /**
     * <p>Invokes Main.main(String[] args) method.</p>
     *
     * @throws Exception to JUnit
     */
    public void testCommandLineDemo5() throws Exception {
        // specify output file
        // -o test_files/output.html test_files/a.txt test_files/b.java
        Main.main(new String[] {"-o", "test_files/output.html", "test_files/a.txt", "test_files/b.jav"});
    }

}
