/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.StringWriter;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

import com.topcoder.util.config.Property;

/**
 * <p> Helper methods for tests. </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class TestHelper {

    /** Private constructor to prevent instantiation. */
    private TestHelper() {
        // private
    }

    /**
     * <p>Reads file contents into a string (with default charset).</p>
     *
     * @param file a string with file name
     * @return file contents
     * @throws RuntimeException if an exception occurs
     */
    public static String readFile(String file) {
        try {
            StringWriter writer = new StringWriter();
            Reader reader = new BufferedReader(new FileReader(file));
            try {
                int c;
                while ((c = reader.read()) != -1) {
                    writer.write(c);
                }
                return writer.toString();
            } finally {
                reader.close();
            }
        } catch (IOException e) {
            // wrap as unchecked exception
            throw new RuntimeException(e);
        }
    }

    /**
     * <p>Writes given content to a file with specified name.</p>
     *
     * @param content String to be written to a file
     * @param file String filename of the file to be written to.
     * @throws RuntimeException if an exception occurs
     */
    public static void writeFile(String content, String file) {
        try {
            FileWriter writer = new FileWriter(file);
            writer.write(content);
            writer.close();
        } catch (IOException e) {
            // wrap as unchecked exception
            throw new RuntimeException(e);
        }
    }

    /**
     * Obtains the value of specified declared field of given type in given object. This method circumvents private
     * access modifiers by Java reflection.
     *
     * @param object an Object which field value is to be obtained
     * @param type a Class to specify declared field
     * @param fieldName a String with declared field name
     * @return the value of specified declared field of given type in given object
     * @throws RuntimeException if an exception occurs
     */
    public static Object getFieldValue(Object object, Class type, String fieldName) {
        try {
            Field field = type.getDeclaredField(fieldName);
            field.setAccessible(true);
            Object result = field.get(object);
            field.setAccessible(false);
            return result;
        } catch (Exception e) {
            // wrap as unchecked exception
            throw new RuntimeException(e);
        }
    }

    /**
     * Adds subproperty to the given property. Helper method that uses the Java reflection in order to hack into
     * Property class.
     *
     * @param property Property to add subproperty to
     * @param subproperty Property to be added
     */
    public static void addSubproperty(Property property, Property subproperty) {
        try {
            // request the method Property.addProperty(Property) accessible and invoke it by reflection
            Method method = Property.class.getDeclaredMethod("addProperty", new Class[] {Property.class});
            method.setAccessible(true);
            try {
                method.invoke(property, new Object[] {subproperty});
            } finally {
                method.setAccessible(false);
            }
        } catch (Exception e) {
            // wrap as unchecked exception
            throw new RuntimeException(e);
        }
    }

}
