/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.matcher;

import java.io.File;

import junit.framework.TestCase;

import com.topcoder.file.statistics.FileMatchingException;

/**
 * <p>Unit test case for FileNameMatcher class.</p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class FileNameMatcherTests extends TestCase {

    /** A concrete subclass of FileNameMatcher. */
    private static class MyFileNameMatcher extends FileNameMatcher {

        /** The last filename passed to match(String) method. */
        private String filename;

        /**
         * Stores the filename in instance field and return true if filename length is even.
         *
         * @param filename the file name to match.
         * @return if filename length is even.
         */
        protected boolean match(String filename) {
            this.filename = filename;
            return filename.length() % 2 == 0;
        }

        /**
         * Getter for 'filename' field.
         *
         * @return last filename passed to match(String) method
         */
        public String getFilename() {
            return filename;
        }

    }

    /** The matcher instance that is tested. */
    private static final MyFileNameMatcher MATCHER = new MyFileNameMatcher();

    /**
     * Creation test for FileNameMatcher default constructor.
     */
    public void testFileNameMatcher1() {
        new MyFileNameMatcher();
        // nothing to check
    }

    /**
     * Tests match(File) method with null argument. NullPointerException should be thrown.
     *
     * @throws FileMatchingException to JUnit
     */
    public void testMatch1() throws FileMatchingException {
        try {
            MATCHER.match((File) null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests match(File) method with valid arguments.
     *
     * @throws FileMatchingException to JUnit
     */
    public void testMatch2() throws FileMatchingException {
        assertTrue("file name with even length should be matched", MATCHER.match(new File("")));
        assertEquals("file name was not processed properly", MATCHER.getFilename(), "");
        assertFalse("file name with even length should be matched", MATCHER.match(new File("foo.bar")));
        assertEquals("file name was not processed properly", MATCHER.getFilename(), "foo.bar");

        assertTrue("file name with even length should be matched", MATCHER.match(new File("test_files/")));
        assertEquals("file name was not processed properly", MATCHER.getFilename(), "test_files");
        assertFalse("file name with even length should be matched", MATCHER.match(new File("test_files/foo.bar")));
        assertEquals("file name was not processed properly", MATCHER.getFilename(), "foo.bar");
    }

}
