/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics;

import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;

/**
 * <p>Unit test case for StatsResult class.</p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class StatsResultTests extends TestCase {

    /** A metric used in tests. */
    private static final Metric METRIC = new Metric("name", "test display name", "test description");

    /** Another metric used in tests. */
    private static final Metric METRIC_2 = new Metric("name2", "test display name", "test description");

    /** The instance for tests. */
    private StatsResult stats;

    /**
     * Sets up the test. Initializes tested instance.
     */
    protected void setUp() {
        stats = new StatsResult();
        stats.setStat(METRIC, 100);
    }

    /**
     * Creation test for StatsResult default constructor.
     */
    public void testStatsResult1() {
        StatsResult result = new StatsResult();
        assertTrue("new stats result should be empty", result.getMetrics().isEmpty());
    }

    /**
     * Creation test for StatsResult with null argument. NullPointerException should be thrown.
     */
    public void testMetric2() {
        try {
            new StatsResult(null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for StatsResult copy constructor. A new instance of stats with the same data should be
     * created.
     */
    public void testMetric3() {
        StatsResult newStats = new StatsResult(stats);
        assertEquals("new stats metrics should be equal to old one", newStats.getMetrics(), stats.getMetrics());
        assertEquals("new stats values should be equal to old one", newStats.getStat(METRIC), stats
                .getStat(METRIC));

        // verify that change in new stats does not affect old stats
        newStats.setStat(METRIC_2, 200);
        assertEquals("stats metrics should remain unaffected", stats.getMetrics(), Arrays
                .asList(new Object[] {METRIC}));
    }

    /**
     * Tests setStat method with null argument. NullPointerException should be thrown.
     */
    public void testSetStat1() {
        try {
            stats.setStat(null, 0);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests setStat method with valid arguments.
     */
    public void testSetStat2() {
        stats.setStat(METRIC, 10);
        assertEquals("stat was not updated correctly", stats.getStat(METRIC), 10);
        stats.setStat(METRIC_2, 20);
        assertEquals("stat was not set correctly", stats.getStat(METRIC_2), 20);
    }

    /**
     * Tests getStat method with null argument. NullPointerException should be thrown.
     */
    public void testGetStat1() {
        try {
            stats.getStat(null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests getStat method with non-existing metric. IllegalArgumentException should be thrown.
     */
    public void testGetStat2() {
        try {
            stats.getStat(METRIC_2);
            fail("IllegalArgumentException should be thrown.");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Tests getStat method with existing metric. Stat value should be returned.
     */
    public void testGetStat3() {
        assertEquals("stat value was not retrieved correctly", stats.getStat(METRIC), 100);
    }

    /**
     * Tests getMetrics method.
     */
    public void testGetMetrics() {
        List metrics = stats.getMetrics();
        assertEquals("stat metrics list was retrieved incorrectly", metrics, Arrays.asList(new Object[] {METRIC}));

        // verify that changes in stats are not reflected in the returned list
        stats.setStat(METRIC_2, 200);
        assertEquals("stat metrics list should remain unaffected", metrics, Arrays.asList(new Object[] {METRIC}));
    }

    /**
     * Tests aggregate() method with null argument. NullPointerException should be thrown.
     */
    public void testAggregate1() {
        try {
            stats.aggregate(null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests aggregate() method with valid stats argument. New stats should be added to current stats.
     */
    public void testAggregate2() {
        StatsResult newStats = new StatsResult();
        newStats.setStat(METRIC, 10);
        newStats.setStat(METRIC_2, 20);
        stats.aggregate(newStats);

        assertEquals("stat value was not added correctly when aggregating", stats.getStat(METRIC), 110);
        assertEquals("stat value was not set correctly when aggregating", stats.getStat(METRIC_2), 20);
    }

}
